package com.tp.listener.ejb;

import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.ejb.LocalBean;
import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.jws.WebMethod;
import javax.jws.WebParam;
import javax.jws.WebService;
import javax.jws.soap.SOAPBinding;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import com.tp.listener.core.bo.ListenerBO;
import com.tp.listener.exception.ClickNotFoundException;
import com.tp.listener.to.ClickTO;
import com.tp.to.TxTO;

/**
 * @author Traffic Patrol
 */
@Stateless
@LocalBean
@Path("/listener")
@WebService(endpointInterface = "com.tp.listener.ejb.interfaces.IListener", targetNamespace = "http://ws.listener.tp.com/")
@SOAPBinding(style = SOAPBinding.Style.DOCUMENT, parameterStyle = SOAPBinding.ParameterStyle.WRAPPED)
public class ListenerBean {

	private ListenerBO listenerBO = null;

    @PostConstruct
    public void postConstruct() {

        if (this.listenerBO == null) {
            this.listenerBO = new ListenerBO();
        }
    }
    
	private synchronized ListenerBO getListenerBO() {
	    return this.listenerBO;
	}

	private TxTO validateTxTO(TxTO txTO) {
		if(txTO == null) {
			txTO = new TxTO();
			txTO.setTxUser("rest");
			txTO.setTxDate(new Date());
		}
		return txTO;
	}

	@WebMethod
	public ClickTO insertClick(String clickDoc, TxTO txTO) {
        ClickTO clickTO = new ClickTO(clickDoc);
        return this.getListenerBO().insertClick(clickTO, txTO);
	}
	
	@POST
    @Path("/insertClick")
    @Consumes( { MediaType.APPLICATION_JSON, MediaType.APPLICATION_XML })
    @Produces( { MediaType.APPLICATION_JSON })
	public ClickTO insertClick(
			@WebParam(name = "clickId") String clickDoc, 
			@WebParam(name = "txTO") TxTO txTO, 
			@Context HttpServletRequest req) {
        ClickTO clickTO = new ClickTO(clickDoc);
        if(req != null) {
        	clickTO.setIp(req.getRemoteAddr());
        	clickTO.setHost(req.getRemoteHost());
        	clickTO.setPort(req.getRemotePort());
        }
        return this.getListenerBO().insertClick(clickTO, validateTxTO(txTO));
	}

	
	/**
	 * Finds an Click by id
	 * 
	 * @param clickId
	 *            - the Click id
	 * @return ClickTO - the account
	 * @throws ClickNotFoundException
	 */
	@WebMethod
	@TransactionAttribute(value = TransactionAttributeType.NOT_SUPPORTED)
	@GET
    @Path("/getClickById")
    @Produces( { MediaType.APPLICATION_JSON })
	public ClickTO getClickById(
			@WebParam(name = "clickId") @QueryParam("clickId") String clickId, 
			@WebParam(name = "txTO") TxTO txTO) throws ClickNotFoundException {
		return this.getListenerBO().getClickById(clickId, validateTxTO(txTO));
	}

	/**
	 * @param externalId
	 * @param atCounter
	 * @return
	 * @throws ClickNotFoundException
	 */
	@WebMethod
	@TransactionAttribute(value = TransactionAttributeType.NOT_SUPPORTED)
    @GET
    @Path("/getClickListByCampaign")
    @Produces( { MediaType.APPLICATION_JSON })
	public List<ClickTO> getClickListByCampaign(
			@WebParam(name = "campaignId") @QueryParam("campaignId") Long campaignId, 
			@WebParam(name = "txTO") TxTO txTO) {
		return this.getListenerBO().getClickListByCampaign(campaignId, validateTxTO(txTO));
    }
}
